@@ -109,10 +109,10 @@ QT_BEGIN_NAMESPACE
 
 QEglFSIntegration::QEglFSIntegration()
     : m_display(EGL_NO_DISPLAY),
-      m_inputContext(0),
+      m_inputContext(nullptr),
       m_fontDb(new QGenericUnixFontDatabase),
       m_services(new QGenericUnixServices),
-      m_kbdMgr(0),
+      m_kbdMgr(nullptr),
       m_disableInputHandlers(false)
 {
     m_disableInputHandlers = qEnvironmentVariableIntValue("QT_QPA_EGLFS_DISABLE_INPUT");
@@ -223,7 +223,7 @@ QPlatformOpenGLContext *QEglFSIntegration::createPlatformOpenGLContext(QOpenGLCo
         EGLConfig config = QEglFSDeviceIntegration::chooseConfig(dpy, adjustedFormat);
         ctx = new QEglFSContext(adjustedFormat, share, dpy, &config, QVariant());
     } else {
-        ctx = new QEglFSContext(adjustedFormat, share, dpy, 0, nativeHandle);
+        ctx = new QEglFSContext(adjustedFormat, share, dpy, nullptr, nativeHandle);
     }
     nativeHandle = QVariant::fromValue<QEGLNativeContext>(QEGLNativeContext(ctx->eglContext(), dpy));
 
@@ -236,7 +236,7 @@ QPlatformOffscreenSurface *QEglFSIntegration::createPlatformOffscreenSurface(QOf
     EGLDisplay dpy = surface->screen() ? static_cast<QEglFSScreen *>(surface->screen()->handle())->display() : display();
     QSurfaceFormat fmt = qt_egl_device_integration()->surfaceFormatFor(surface->requestedFormat());
     if (qt_egl_device_integration()->supportsPBuffers()) {
-        QEGLPlatformContext::Flags flags = 0;
+        QEGLPlatformContext::Flags flags;
         if (!qt_egl_device_integration()->supportsSurfacelessContexts())
             flags |= QEGLPlatformContext::NoSurfaceless;
         return new QEGLPbuffer(dpy, fmt, surface, flags);
@@ -247,6 +247,13 @@ QPlatformOffscreenSurface *QEglFSIntegration::createPlatformOffscreenSurface(QOf
 }
 #endif // QT_NO_OPENGL
 
+#if QT_CONFIG(vulkan)
+QPlatformVulkanInstance *QEglFSIntegration::createPlatformVulkanInstance(QVulkanInstance *instance) const
+{
+    return qt_egl_device_integration()->createPlatformVulkanInstance(instance);
+}
+#endif
+
 bool QEglFSIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 {
     // We assume that devices will have more and not less capabilities
@@ -283,7 +290,8 @@ enum ResourceType {
     NativeDisplay,
     XlibDisplay,
     WaylandDisplay,
-    EglSurface
+    EglSurface,
+    VkSurface
 };
 
 static int resourceType(const QByteArray &key)
@@ -296,7 +304,8 @@ static int resourceType(const QByteArray &key)
         QByteArrayLiteral("nativedisplay"),
         QByteArrayLiteral("display"),
         QByteArrayLiteral("server_wl_display"),
-        QByteArrayLiteral("eglsurface")
+        QByteArrayLiteral("eglsurface"),
+        QByteArrayLiteral("vksurface")
     };
     const QByteArray *end = names + sizeof(names) / sizeof(names[0]);
     const QByteArray *result = std::find(names, end, key);
@@ -307,7 +316,7 @@ static int resourceType(const QByteArray &key)
 
 void *QEglFSIntegration::nativeResourceForIntegration(const QByteArray &resource)
 {
-    void *result = 0;
+    void *result = nullptr;
 
     switch (resourceType(resource)) {
     case EglDisplay:
@@ -329,7 +338,7 @@ void *QEglFSIntegration::nativeResourceForIntegration(const QByteArray &resource
 
 void *QEglFSIntegration::nativeResourceForScreen(const QByteArray &resource, QScreen *screen)
 {
-    void *result = 0;
+    void *result = nullptr;
 
     switch (resourceType(resource)) {
     case XlibDisplay:
@@ -347,7 +356,7 @@ void *QEglFSIntegration::nativeResourceForScreen(const QByteArray &resource, QSc
 
 void *QEglFSIntegration::nativeResourceForWindow(const QByteArray &resource, QWindow *window)
 {
-    void *result = 0;
+    void *result = nullptr;
 
     switch (resourceType(resource)) {
     case EglDisplay:
@@ -364,6 +373,12 @@ void *QEglFSIntegration::nativeResourceForWindow(const QByteArray &resource, QWi
         if (window && window->handle())
             result = reinterpret_cast<void*>(static_cast<QEglFSWindow *>(window->handle())->surface());
         break;
+#if QT_CONFIG(vulkan)
+    case VkSurface:
+        if (window && window->handle() && window->surfaceType() == QSurface::VulkanSurface)
+            result = static_cast<QEglFSWindow *>(window->handle())->vulkanSurfacePtr();
+        break;
+#endif
     default:
         break;
     }
@@ -374,7 +389,7 @@ void *QEglFSIntegration::nativeResourceForWindow(const QByteArray &resource, QWi
 #ifndef QT_NO_OPENGL
 void *QEglFSIntegration::nativeResourceForContext(const QByteArray &resource, QOpenGLContext *context)
 {
-    void *result = 0;
+    void *result = nullptr;
 
     switch (resourceType(resource)) {
     case EglContext:
@@ -402,7 +417,7 @@ static void *eglContextForContext(QOpenGLContext *context)
 
     QEglFSContext *handle = static_cast<QEglFSContext *>(context->handle());
     if (!handle)
-        return 0;
+        return nullptr;
 
     return handle->eglContext();
 }
@@ -416,7 +431,7 @@ QPlatformNativeInterface::NativeResourceForContextFunction QEglFSIntegration::na
 #else
     Q_UNUSED(resource);
 #endif
-    return 0;
+    return nullptr;
 }
 
 QFunctionPointer QEglFSIntegration::platformFunction(const QByteArray &function) const
